@@ -15,7 +15,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this library. If not, see <https://www.gnu.org/licenses/>.
-'''Attributors are convienence objects to compute an attributions, optionally using composites.'''
+'''Attributors are convenience objects to compute attributions, optionally using composites.'''
 from abc import ABCMeta, abstractmethod
 from functools import partial
 from itertools import product
@@ -24,16 +24,64 @@ import torch
 
 
 def constant(obj):
+    '''Wrapper function to create a function which returns a constant object regardless of arguments.
+
+    Parameters
+    ----------
+    obj : object
+        Constant object which the returned wrapper function will return on call.
+
+    Returns
+    -------
+    wrapped_const : function
+        Function which when called with any arguments will return ``obj``.
+
+    '''
     def wrapped_const(*args, **kwargs):
         return obj
     return wrapped_const
 
 
 def identity(obj):
+    '''Identity function.
+
+    Parameters
+    ----------
+    obj : object
+        Any object which will be returned.
+
+    Result
+    ------
+    obj : object
+        The original input argument ``obj``.
+
+    '''
     return obj
 
 
 def occlude_independent(input, mask, fill_fn=torch.zeros_like, invert=False):
+    '''Given a ``mask``, occlude pixels of ``input`` independently given a function ``fill_fn``.
+
+    Parameters
+    ----------
+    input : :py:obj:`torch.Tensor`
+        The input tensor which will be occluded in the pixels where ``mask`` is non-zero, or, if ``invert`` is
+        ``True``, where ``mask`` is zero, using function ``fill_fn``.
+    mask : :py:obj:`torch.Tensor`
+        Boolean mask, at which non-zero or zero (given ``invert``) elements will be occluded in ``input``.
+    fill_fn : function, optional
+        Function used to occlude pixels with signature ``(input : torch.Tensor) -> torch.Tensor``, where input is the
+        same shape as ``input``, and the output shall leave the shape unchanged. Default is ``torch.zeros_like``, which
+        will replace occluded pixels with zero.
+    invert : bool, optional
+        If ``True``, inverts the supplied mask. Default is ``False``, i.e. not to invert.
+
+    Returns
+    -------
+    :py:obj:`torch.Tensor`
+        The occluded tensor.
+
+    '''
     if invert:
         mask = ~mask
     return input * mask + ~mask * fill_fn(input)
@@ -42,7 +90,7 @@ def occlude_independent(input, mask, fill_fn=torch.zeros_like, invert=False):
 class Attributor(metaclass=ABCMeta):
     '''Base Attributor Class.
 
-    Attributors are convienience objects with an optional composite and when called, compute an attribution, e.g., the
+    Attributors are convenience objects with an optional composite and when called, compute an attribution, e.g., the
     gradient or anything that is the result of computing the gradient when using the provided composite.  Attributors
     also provide a context to be used in a `with` statement, similar to `CompositeContext`s. If the forward function
     (or `self.__call__`) is called and the composite has not been registered (i.e. `composite.handles` is empty), the
@@ -150,8 +198,9 @@ class Attributor(metaclass=ABCMeta):
 
 
 class Gradient(Attributor):
-    '''The Gradient Attributor. The result is the product of the attribution output and the (possibly modified) jacobian.
-    With a composite, i.e. `EpsilonGammaBox`, this will compute the Layerwise Relevance Propagation attribution values.
+    '''The Gradient Attributor. The result is the product of the attribution output and the (possibly modified)
+    jacobian. With a composite, i.e. `EpsilonGammaBox`, this will compute the Layerwise Relevance Propagation
+    attribution values.
     '''
     def forward(self, input, attr_output_fn):
         '''Compute the gradient of the model wrt. input, by using `attr_output_fn` as the function of the model output
@@ -278,8 +327,8 @@ class IntegratedGradients(Attributor):
     References
     ----------
     .. [2] M. Sundararajan, A. Taly, and Q. Yan, “Axiomatic attribution for deep networks,” in Proceedings of the 34th
-       International Conference on Machine Learning, ICML 2017, Sydney, NSW, Australia, 6-11 August 2017, ser. Proceedings
-       of Machine Learning Research, D. Precup and Y. W. Teh, Eds., vol. 70. PMLR, 2017, pp. 3319–3328.
+       International Conference on Machine Learning, ICML 2017, Sydney, NSW, Australia, 6-11 August 2017, ser.
+       Proceedings of Machine Learning Research, D. Precup and Y. W. Teh, Eds., vol. 70. PMLR, 2017, pp. 3319–3328.
 
     '''
     def __init__(self, model, composite=None, attr_output=None, baseline_fn=None, n_iter=20):
@@ -346,10 +395,11 @@ class Occlusion(Attributor):
         default, all values except inside the sliding window will be occluded.
     window: int or tuple of ints, optional
         The size of the sliding window to occlude over the input for each dimension. Defaults to 8. If a single integer
-        is provided, the sliding window will slide over all dimensions with the same size. If a tuple is provided, the
-        window will only slide over the n-last dimensions, where n is the length of the tuple, e.g., if the data has
-        shape `(3, 32, 32)` and `window=(8, 8)`, the resulting mask will have a block of shape `(3, 8, 8)` set to True.
-        `window` must have the same length as `stride`.
+        is provided, the sliding window will slide with the same size over all dimensions except the first, which is
+        assumed as the batch-dimension. If a tuple is provided, the window will only slide over the n-last dimensions,
+        where n is the length of the tuple, e.g., if the data has shape `(3, 32, 32)` and `window=(8, 8)`, the
+        resulting mask will have a block of shape `(3, 8, 8)` set to True. `window` must have the same length as
+        `stride`.
     stride: int or tuple of ints, optional
         The strides used for the sliding window to occlude over the input for each dimension. Defaults to 8. If a
         single integer is provided, the strides will be the same size for all dimensions. If a tuple is provided,
@@ -378,9 +428,9 @@ class Occlusion(Attributor):
         window = self.window
         stride = self.stride
         if isinstance(window, int):
-            window = tuple(min(window, size) for size in input.shape)
+            window = tuple(min(window, size) for size in input.shape[1:])
         if isinstance(stride, int):
-            window = tuple(min(stride, size) for size in input.shape)
+            stride = tuple(min(stride, size) for size in input.shape[1:])
 
         if len(window) < input.ndim:
             window = tuple(input.shape)[:input.ndim - len(window)] + window
