@@ -24,23 +24,74 @@ import torch
 
 
 def stabilize(input, epsilon=1e-6):
-    '''Stabilize input for safe division.
+    '''Stabilize input for safe division. This shifts zero-elements by ``+ epsilon``. For the sake of the
+    *epsilon rule*, this also shifts positive values by ``+ epsilon`` and negative values by ``- epsilon``.
 
     Parameters
     ----------
-    input: obj:`torch.Tensor`
+    input: :py:obj:`torch.Tensor`
         Tensor to stabilize.
     epsilon: float, optional
-        Value to replace zero elements with.
+        Value by which to shift elements.
 
     Returns
     -------
-    obj:`torch.Tensor`
-        New Tensor copied from `input` with all zero elements set to epsilon.
+    :py:obj:`torch.Tensor`
+        New Tensor copied from `input` with values shifted by epsilon.
     '''
     return input + ((input == 0.).to(input) + input.sign()) * epsilon
 
 
+def expand(tensor, shape, cut_batch_dim=False):
+    '''Expand a scalar value or tensor to a shape. In addition to torch.Tensor.expand, this will also accept
+    non-torch.tensor objects, which will be used to create a new tensor. If ``tensor`` has fewer dimensions than
+    ``shape``, singleton dimension will be appended to match the size of ``shape`` before expanding.
+
+    Parameters
+    ----------
+    tensor : int, float or :py:obj:`torch.Tensor`
+        Scalar or tensor to expand to the size of ``shape``.
+    shape : tuple[int]
+        Shape to which ``tensor`` will be expanded.
+    cut_batch_dim : bool, optional
+        If True, take only the first ``shape[0]`` entries along dimension 0 of the expanded ``tensor``, if it has more
+        entries in dimension 0 than ``shape``. Default (False) is not to cut, which will instead cause a
+        ``RuntimeError`` due to the size mismatch.
+
+    Returns
+    -------
+    :py:obj:`torch.Tensor`
+        A new tensor expanded from ``tensor`` with shape ``shape``.
+
+    Raises
+    ------
+    RuntimeError
+        If ``tensor`` could not be expanded to ``shape`` due to incompatible shapes.
+
+    '''
+    if not isinstance(tensor, torch.Tensor):
+        # cast non-tensor scalar to 0-dim tensor
+        tensor = torch.tensor(tensor)
+    if tensor.ndim == 0:
+        # expand scalar tensors
+        return tensor.expand(shape)
+    if tensor.ndim < len(shape) and all(left in (1, right) for left, right in zip(tensor.shape, shape)):
+        # append singleton dimensions if tensor has fewer dimensions, and the existing ones match with shape
+        tensor = tensor[(...,) + (None,) * (len(shape) - len(tensor.shape))]
+    if tensor.ndim == len(shape):
+        # if the dims match completely (lenghts match and zipped match), expand normally
+        if all(left in (1, right) for left, right in zip(tensor.shape, shape)):
+            return tensor.expand(shape)
+        # if `cut_batch_dim` and dims match except first, which is larger than shape, the the first dim and expand
+        if (
+            cut_batch_dim
+            and all(left in (1, right) for left, right in zip(tensor.shape[1:], shape[1:]))
+            and tensor.shape[0] > shape[0]
+        ):
+            return tensor[:shape[0]].expand(shape)
+    raise RuntimeError('Invalid shape! Target: {tensor.shape}; Source: {shape}')
+
+
 @contextmanager
 def mod_params(module, modifier, param_keys=None, require_params=True):
     '''Context manager to temporarily modify parameter attributes (all by default) of a module.
@@ -56,7 +107,7 @@ def mod_params(module, modifier, param_keys=None, require_params=True):
         A list of parameters that shall be modified. If `None` (default), all parameters are modified (which may be
         none). If `[]`, no parameters are modified and `modifier` is ignored.
     require_params: bool, optional
-        Whether existance of `module`'s params is mandatory (True by default). If the attribute exists but is `None`,
+        Whether existence of `module`'s params is mandatory (True by default). If the attribute exists but is `None`,
         it is not considered missing, and the modifier is not applied.
 
     Raises
@@ -76,7 +127,8 @@ def mod_params(module, modifier, param_keys=None, require_params=True):
 
         missing = [key for key in param_keys if not hasattr(module, key)]
         if require_params and missing:
-            raise RuntimeError('Module {} requires missing parameters: \'{}\''.format(module, '\', \''.join(missing)))
+            missing_str = '\', \''.join(missing)
+            raise RuntimeError(f'Module {module} requires missing parameters: \'{missing_str}\'')
 
         for key in param_keys:
             if key not in missing:
@@ -228,7 +280,7 @@ class BasicHook(Hook):
         A list of parameters that shall be modified. If `None` (default), all parameters are modified (which may be
         none). If `[]`, no parameters are modified and `modifier` is ignored.
     require_params: bool, optional
-        Whether existance of `module`'s params is mandatory (True by default). If the attribute exists but is `None`,
+        Whether existence of `module`'s params is mandatory (True by default). If the attribute exists but is `None`,
         it is not considered missing, and the modifier is not applied.
     '''
     def __init__(
